@@ -56,14 +56,13 @@
 #include "gthumb-info-bar.h"
 #include "gtk-utils.h"
 #include "main.h"
-#include "nav-window.h"
+#include "gth-nav-window.h"
 #include "image-viewer.h"
 #include "jpeg-utils.h"
 #include "preferences.h"
 #include "typedefs.h"
 
 #include "icons/pixbufs.h"
-#include "icons/nav_button.xpm"
 
 #define GLADE_EXPORTER_FILE "gthumb_png_exporter.glade"
 #define DEFAULT_WIN_WIDTH 200
@@ -98,9 +97,6 @@ struct _GthViewerPrivateData {
 	GtkWidget       *image_popup_menu;
 	GtkWidget       *info_bar;
 	GtkWidget       *viewer;
-	GtkWidget       *viewer_vscr;
-	GtkWidget       *viewer_hscr;
-	GtkWidget       *viewer_nav_event_box;
 	GtkWidget       *image_data_hpaned;
 	GtkWidget       *image_comment;
 	GtkWidget       *exif_data_viewer;
@@ -793,16 +789,21 @@ static void
 viewer_update_open_with_menu (GthViewer *viewer)
 {
 	GthViewerPrivateData *priv = viewer->priv;
+	GList                *menu_items, *scan;
 	const char           *mime_type = NULL;
+	GtkWidget            *mitem;
+	int                   pos = 0, i;
 
-	gtk_container_foreach  (GTK_CONTAINER (priv->open_with_popup_menu), (GtkCallback) gtk_widget_destroy, NULL);
-
+	menu_items = gtk_container_get_children (GTK_CONTAINER (priv->open_with_popup_menu));
+	for (i = 0, scan = menu_items; i < g_list_length (menu_items) - 1; i++, scan = scan->next) 
+		gtk_widget_destroy ((GtkWidget*) scan->data);
+	g_list_free (menu_items);
+		
 	if (priv->image_path != NULL)
-		mime_type = gnome_vfs_get_file_mime_type (priv->image_path, NULL, FALSE);
+		mime_type = get_mime_type (priv->image_path);
 
 	if (mime_type != NULL) {
 		GList        *apps = gnome_vfs_mime_get_all_applications (mime_type);
-		GList        *scan;
 		GtkIconTheme *theme = gtk_icon_theme_get_default ();
 		int           icon_size = get_folder_pixbuf_size_for_list (GTK_WIDGET (viewer));
 
@@ -810,6 +811,10 @@ viewer_update_open_with_menu (GthViewer *viewer)
 			GnomeVFSMimeApplication *app = scan->data;
 			GtkWidget               *mitem;
 
+			/* do not include gthumb itself */
+			if (strncmp (gnome_vfs_mime_application_get_exec (app), "gthumb", 6) == 0)
+				continue;
+
 			mitem = gtk_image_menu_item_new_with_label (gnome_vfs_mime_application_get_name (app));
 			gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (mitem), create_image (theme, gnome_vfs_mime_application_get_icon (app), icon_size));
 			g_object_set_data_full (G_OBJECT (mitem), "app", app, (GDestroyNotify)gnome_vfs_mime_application_free);
@@ -817,17 +822,20 @@ viewer_update_open_with_menu (GthViewer *viewer)
 					  G_CALLBACK (open_with_menu_item_activate_cb),
 					  viewer);
 			gtk_widget_show_all (mitem);
-			gtk_menu_append (priv->open_with_popup_menu, mitem);
+			gtk_menu_insert (priv->open_with_popup_menu, mitem, pos++);
 		}
 		g_list_free (apps);
 
 	} else {
-		GtkWidget *mitem;
 		mitem = gtk_menu_item_new_with_label (_("_None"));
 		gtk_widget_set_sensitive (mitem, FALSE);
 		gtk_widget_show (mitem);
-		gtk_menu_append (priv->open_with_popup_menu, mitem);
+		gtk_menu_insert (priv->open_with_popup_menu, mitem, pos++);
 	}
+
+	mitem = gtk_separator_menu_item_new ();
+	gtk_widget_show (mitem);
+	gtk_menu_insert (priv->open_with_popup_menu, mitem, pos++);
 }
 
 
@@ -1238,34 +1246,6 @@ viewer_key_press_cb (GtkWidget   *widget,
 }
 
 
-static gboolean
-size_changed_cb (GtkWidget *widget, 
-		 GthViewer *viewer)
-{
-	GthViewerPrivateData *priv = viewer->priv;
-	GtkAdjustment        *vadj, *hadj;
-	gboolean              hide_vscr, hide_hscr;
-
-	vadj = IMAGE_VIEWER (priv->viewer)->vadj;
-	hadj = IMAGE_VIEWER (priv->viewer)->hadj;
-
-	hide_vscr = vadj->upper <= vadj->page_size;
-	hide_hscr = hadj->upper <= hadj->page_size;
-
-	if (hide_vscr && hide_hscr) {
-		gtk_widget_hide (priv->viewer_vscr); 
-		gtk_widget_hide (priv->viewer_hscr); 
-		gtk_widget_hide (priv->viewer_nav_event_box);
-	} else {
-		gtk_widget_show (priv->viewer_vscr); 
-		gtk_widget_show (priv->viewer_hscr); 
-		gtk_widget_show (priv->viewer_nav_event_box);
-	}
-
-	return TRUE;	
-}
-
-
 static gboolean
 image_focus_changed_cb (GtkWidget     *widget,
 			GdkEventFocus *event,
@@ -1446,7 +1426,7 @@ monitor_file_renamed_cb (GthMonitor *monitor,
 		return;
 
 	g_free (viewer->priv->image_path);
-	viewer->priv->image_path = g_strdup (new_name);
+	viewer->priv->image_path = get_uri_from_path (new_name);
 
 	gth_window_reload_current_image (GTH_WINDOW (viewer));
 }
@@ -1497,7 +1477,7 @@ add_open_with_toolbar_item (GthViewer *viewer)
 		return;
 	}
 
-	priv->open_with_popup_menu = gtk_menu_new ();
+	priv->open_with_popup_menu = gtk_ui_manager_get_widget (priv->ui, "/OpenWithMenu");
 
 	priv->open_with_tool_item = gtk_menu_tool_button_new (gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_LARGE_TOOLBAR), _("_Open With"));
 
@@ -1567,8 +1547,7 @@ gth_viewer_construct (GthViewer   *viewer,
 	GtkWidget            *button;
 	GtkWidget            *image;
 	GtkWidget            *image_vpaned;
-	GtkWidget            *table;
-	GtkWidget            *hbox;
+	GtkWidget            *nav_window;
 	GtkWidget            *scrolled_window;
 	GtkActionGroup       *actions;
 	GtkUIManager         *ui;
@@ -1718,10 +1697,6 @@ gth_viewer_construct (GthViewer   *viewer,
 			  "zoom_changed",
 			  G_CALLBACK (zoom_changed_cb), 
 			  viewer);
-	g_signal_connect (G_OBJECT (priv->viewer), 
-			  "size_changed",
-			  G_CALLBACK (size_changed_cb), 
-			  viewer);
 	g_signal_connect (G_OBJECT (priv->viewer), 
 			  "focus_in_event",
 			  G_CALLBACK (image_focus_changed_cb), 
@@ -1761,15 +1736,7 @@ gth_viewer_construct (GthViewer   *viewer,
 			  window);
 	*/
 
-	priv->viewer_vscr = gtk_vscrollbar_new (IMAGE_VIEWER (priv->viewer)->vadj);
-	priv->viewer_hscr = gtk_hscrollbar_new (IMAGE_VIEWER (priv->viewer)->hadj);
-	priv->viewer_nav_event_box = gtk_event_box_new ();
-	gtk_container_add (GTK_CONTAINER (priv->viewer_nav_event_box), _gtk_image_new_from_xpm_data (nav_button_xpm));
-
-	g_signal_connect (G_OBJECT (priv->viewer_nav_event_box), 
-			  "button_press_event",
-			  G_CALLBACK (nav_button_clicked_cb), 
-			  priv->viewer);
+	nav_window = gth_nav_window_new (GTH_IVIEWER (priv->viewer));
 
 	/* Image comment */
 
@@ -1832,17 +1799,9 @@ gth_viewer_construct (GthViewer   *viewer,
              |
              +- image_vpaned
                   |
-                  +- table
+                  +- nav_window
                   |    |
-                  |    +- hbox
-                  |    |    |
-                  |    |    +- priv->viewer
-                  |    |
-                  |    +- priv->viewer_vscr
-                  |    |
-                  |    +- priv->viewer_hscr
-                  |    |
-                  |    +- priv->image_nav_button_box
+                  |    +- priv->viewer
                   |
                   +- priv->image_data_hpaned
                        |
@@ -1872,25 +1831,6 @@ gth_viewer_construct (GthViewer   *viewer,
 	priv->image_main_pane = image_vpaned = gtk_vpaned_new ();
 	gtk_paned_set_position (GTK_PANED (viewer->priv->image_main_pane), eel_gconf_get_integer (PREF_UI_VIEWER_HEIGHT, DEFAULT_WIN_HEIGHT) - eel_gconf_get_integer (PREF_UI_COMMENT_PANE_SIZE, DEFAULT_COMMENT_PANE_SIZE));
 
-	/* ** table */
-
-	hbox = gtk_hbox_new (FALSE, 0);
-	gtk_container_add (GTK_CONTAINER (hbox), priv->viewer);
-
-	table = gtk_table_new (2, 2, FALSE);
-	gtk_table_attach (GTK_TABLE (table), hbox, 0, 1, 0, 1,
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), priv->viewer_vscr, 1, 2, 0, 1,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), priv->viewer_hscr, 0, 1, 1, 2,
-			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-	gtk_table_attach (GTK_TABLE (table), priv->viewer_nav_event_box, 1, 2, 1, 2,
-			  (GtkAttachOptions) (GTK_FILL),
-			  (GtkAttachOptions) (GTK_FILL), 0, 0);
-
 	/* ** priv->image_data_hpaned */
 
 	priv->image_data_hpaned = gtk_hpaned_new ();
@@ -1906,7 +1846,7 @@ gth_viewer_construct (GthViewer   *viewer,
 
 	/**/
 
-	gtk_paned_pack1 (GTK_PANED (image_vpaned), table, FALSE, FALSE);
+	gtk_paned_pack1 (GTK_PANED (image_vpaned), nav_window, FALSE, FALSE);
 	gtk_paned_pack2 (GTK_PANED (image_vpaned), priv->image_data_hpaned, TRUE, FALSE);
 
 	gtk_box_pack_start (GTK_BOX (image_vbox), info_frame, FALSE, FALSE, 0);
@@ -2012,7 +1952,7 @@ gth_viewer_construct (GthViewer   *viewer,
 	/**/
 
 	if (filename != NULL)
-		priv->image_path = g_strdup (filename);
+		priv->image_path = get_uri_from_path (filename);
 }
 
 
@@ -2044,7 +1984,7 @@ load_image__image_saved_cb (const char *filename,
 
 	if (priv->new_image_path != priv->image_path) {
 		g_free (priv->image_path);
-		priv->image_path = g_strdup (priv->new_image_path);
+		priv->image_path = get_uri_from_path (priv->new_image_path);
 	}
 
 	image_viewer_load_image (IMAGE_VIEWER (priv->viewer), priv->image_path);
@@ -2061,7 +2001,7 @@ gth_viewer_load (GthViewer   *viewer,
 	g_free (priv->new_image_path);
 	priv->new_image_path = NULL;
 	if (filename != NULL)
-		priv->new_image_path = g_strdup (filename);
+		priv->new_image_path = get_uri_from_path (filename);
 
 	if (priv->image_modified) {
 		if (priv->saving_modified_image)
