@@ -29,19 +29,20 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libbonobo.h>
 
-#include "auto-completion.h"
 #include "catalog.h"
 #include "comments.h"
 #include "dir-list.h"
 #include "main.h"
 #include "file-utils.h"
 #include "gconf-utils.h"
+#include "gtk-utils.h"
 #include "gth-application.h"
 #include "gth-browser.h"
 #include "gth-browser-actions-callbacks.h"
 #include "gth-image-list.h"
 #include "gth-viewer.h"
 #include "gthumb-init.h"
+#include "gthumb-stock.h"
 #include "image-viewer.h"
 #include "pixbuf-utils.h"
 #include "preferences.h"
@@ -49,7 +50,18 @@
 
 #include "icons/pixbufs.h"
 
-#define ICON_NAME_DIRECTORY "gnome-fs-directory"
+typedef enum {
+	ICON_NAME_DIRECTORY,
+	ICON_NAME_HARDDISK,
+	ICON_NAME_HOME,
+	ICON_NAME_DESKTOP,
+	ICON_NAMES
+} IconName;
+
+static char*icon_mime_name[ICON_NAMES] = { "gnome-fs-directory",
+					   "gnome-dev-harddisk",
+					   "gnome-fs-home",
+					   "gnome-fs-desktop" };
 
 GthMonitor  *monitor = NULL;
 char       **file_urls, **dir_urls;
@@ -65,7 +77,7 @@ gboolean     ImportPhotos = FALSE;
 
 static gboolean        view_comline_catalog = FALSE;
 static gboolean        view_single_image = FALSE;
-static GdkPixbuf      *folder_pixbuf = NULL;
+static GdkPixbuf      *icon_pixbuf[ICON_NAMES] = { 0 };
 static GtkWidget      *first_window = NULL;
 static GnomeIconTheme *icon_theme = NULL;
 static BonoboObject   *gth_application = NULL;
@@ -84,14 +96,32 @@ struct poptOption options[] = {
 };
 
 
+static void
+init_icon_pixbufs (void)
+{
+	int i;
+	for (i = 0; i < ICON_NAMES; i++) 
+		icon_pixbuf[i] = NULL;
+}
+
+
+static void
+free_icon_pixbufs (void)
+{
+	int i;
+	for (i = 0; i < ICON_NAMES; i++)
+		if (icon_pixbuf[i] != NULL) {
+			g_object_unref (icon_pixbuf[i]);
+			icon_pixbuf[i] = NULL;
+		}
+}
+
+
 static void
 theme_changed_cb (GnomeIconTheme *theme, 
 		  gpointer        data)
 {
-	if (folder_pixbuf != NULL) {
-		g_object_unref (folder_pixbuf);
-		folder_pixbuf = NULL;
-	}
+	free_icon_pixbufs ();
 	all_windows_notify_update_icon_theme ();
 }
 
@@ -200,8 +230,7 @@ save_session (GnomeClient *client)
 			location = gth_window_get_image_filename (window);
 			if (location == NULL)
 				continue;
-
-			uri = g_strconcat ("file://", location, NULL);
+			uri = get_uri_from_path (location);
 
 		} else {
 			GthBrowser *browser = (GthBrowser*) window;
@@ -211,14 +240,14 @@ save_session (GnomeClient *client)
 				location = gth_browser_get_current_directory (browser);
 				if (location == NULL)
 					continue;
-				uri = g_strconcat ("file://", location, NULL);
+				uri = get_uri_from_path (location);
 				break;
 			
 			case GTH_SIDEBAR_CATALOG_LIST:
 				location = gth_browser_get_current_catalog (browser);
 				if (location == NULL)
 					continue;
-				uri = g_strconcat ("file://", location, NULL);
+				uri = get_uri_from_path (location);
 				break;
 				
 			default:
@@ -356,6 +385,7 @@ initialize_data (poptContext pctx)
 	/* Default windows icon */
 
 	gtk_window_set_default_icon_name ("gthumb");
+	init_icon_pixbufs ();
 
 	/**/
 
@@ -414,9 +444,7 @@ initialize_data (poptContext pctx)
 		}
 
 		if (is_dir) {
-			dir_urls[n_dir_urls++] = g_strconcat ("file://",
-							      path,
-							      NULL);
+			dir_urls[n_dir_urls++] = get_uri_from_path (path);
 			g_free (path);
 		} else
 			file_urls[n_file_urls++] = path;
@@ -464,8 +492,7 @@ release_data (void)
 	if (gth_application != NULL)
                 bonobo_object_unref (gth_application);
 
-	if (folder_pixbuf != NULL)
-		g_object_unref (folder_pixbuf);
+	free_icon_pixbufs ();
 
 	g_object_unref (icon_theme);
 
@@ -476,7 +503,6 @@ release_data (void)
 
 	preferences_release ();
 	eel_global_client_free ();
-	auto_compl_reset ();
 }
 
 
@@ -543,7 +569,7 @@ load_session (gboolean                  use_factory,
 		key = g_strdup_printf ("Session/location%d", i);
 		location = gnome_config_get_string (key);
 
-		if (pref_util_location_is_file (location) && path_is_file (location))
+		if (uri_scheme_is_file (location) && path_is_file (location))
 			open_viewer_window (location, use_factory, app, env);
 		else
 			open_browser_window (location, TRUE, use_factory, app, env);
@@ -833,6 +859,13 @@ all_windows_notify_catalog_delete (const gchar *path)
 }
 
 
+void
+all_windows_notify_catalog_reordered (const gchar *path)
+{
+	gth_monitor_notify_update_catalog (monitor, path, GTH_MONITOR_EVENT_CHANGED);
+}
+
+
 void 
 all_windows_notify_update_metadata (const gchar *filename)
 {
@@ -861,31 +894,46 @@ all_windows_add_monitor (void)
 }
 
 
-GdkPixbuf *
-get_folder_pixbuf (double icon_size)
+static GdkPixbuf *
+get_fs_icon (IconName icon_name,
+	     double   icon_size)
 {
-	GdkPixbuf       *pixbuf = NULL;
-	static gboolean  scale = TRUE;
+	GdkPixbuf *pixbuf = NULL;
+	gboolean   scale = TRUE;
 
-	if (folder_pixbuf == NULL) {
-		char *icon_path;
+	/*
+	if (icon_pixbuf[icon_name] != NULL) {
+		g_object_ref (icon_pixbuf[icon_name]);
+		return icon_pixbuf[icon_name];
+	}
+
+	icon_pixbuf[icon_name] = create_pixbuf (icon_theme,
+						icon_mime_name[icon_name],
+						icon_size);
+	
+	return icon_pixbuf[icon_name];
+	*/
+
+	if (icon_pixbuf[icon_name] == NULL) {
+		const GnomeIconData *icon_data;
+		int                  base_size;
+		char                *icon_path;
 		
 		icon_path = gnome_icon_theme_lookup_icon (icon_theme,
-							  ICON_NAME_DIRECTORY,
+							  icon_mime_name[icon_name],
 							  icon_size,
-							  NULL,
-							  NULL);
+							  &icon_data,
+							  &base_size);
 
 		if (icon_path == NULL) {
-			folder_pixbuf = gdk_pixbuf_new_from_inline (-1, 
-								    dir_16_rgba, 
-								    FALSE, 
-								    NULL);
+			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_inline (-1, 
+									     dir_16_rgba, 
+									     FALSE, 
+									     NULL);
 			scale = FALSE;
-
 		} else {
-			folder_pixbuf = gdk_pixbuf_new_from_file (icon_path, 
-								  NULL);
+			icon_pixbuf[icon_name] = gdk_pixbuf_new_from_file (icon_path, 
+									   NULL);
 			g_free (icon_path);
 		}
 	}
@@ -893,22 +941,22 @@ get_folder_pixbuf (double icon_size)
 	/* Scale keeping aspect ratio. */
 
  	if (! scale) {
-		g_object_ref (folder_pixbuf);
-		return folder_pixbuf;
+		g_object_ref (icon_pixbuf[icon_name]);
+		return icon_pixbuf[icon_name];
 	}
-	
-	if (folder_pixbuf != NULL) {
+
+	if (icon_pixbuf[icon_name] != NULL) {
 		int w, h;
 		
-		w = gdk_pixbuf_get_width (folder_pixbuf);
-		h = gdk_pixbuf_get_height (folder_pixbuf);
+		w = gdk_pixbuf_get_width (icon_pixbuf[icon_name]);
+		h = gdk_pixbuf_get_height (icon_pixbuf[icon_name]);
 		if (scale_keepping_ratio (&w, &h, icon_size, icon_size)) 
-			pixbuf = gdk_pixbuf_scale_simple (folder_pixbuf,
+			pixbuf = gdk_pixbuf_scale_simple (icon_pixbuf[icon_name],
 							  w,
 							  h,
 							  GDK_INTERP_BILINEAR);
 		else {
-			pixbuf = folder_pixbuf;
+			pixbuf = icon_pixbuf[icon_name];
 			g_object_ref (pixbuf);
 		}
 	}
@@ -917,6 +965,13 @@ get_folder_pixbuf (double icon_size)
 }
 
 
+GdkPixbuf *
+get_folder_pixbuf (double icon_size)
+{
+	return get_fs_icon (ICON_NAME_DIRECTORY, icon_size);
+}
+
+
 int
 get_folder_pixbuf_size_for_list (GtkWidget *widget)
 {
@@ -947,7 +1002,7 @@ folder_is_film (const char *folder)
 	CommentData *cdata;
 	gboolean     film = FALSE;
 
-	folder = pref_util_remove_prefix (folder);
+	folder = remove_scheme_from_uri (folder);
 
 	cdata = comments_load_comment (folder, FALSE);
 	if (cdata != NULL) {
@@ -962,3 +1017,64 @@ folder_is_film (const char *folder)
 
 	return film;
 }
+
+
+G_CONST_RETURN char *
+get_stock_id_for_uri (const char *uri)
+{
+	const char *stock_id;
+	char       *home_uri;
+
+	home_uri = g_strconcat ("file://", g_get_home_dir (), NULL);
+
+	if (strcmp (uri, g_get_home_dir ()) == 0) 
+		stock_id = GTK_STOCK_HOME;
+	else if (strcmp (uri, home_uri) == 0) 
+		stock_id = GTK_STOCK_HOME;
+	else if (folder_is_film (uri))
+		stock_id = GTHUMB_STOCK_FILM;
+	else if (uri_scheme_is_catalog (uri)) 
+		stock_id = GTHUMB_STOCK_CATALOG;
+	else if (uri_scheme_is_search (uri))	
+		stock_id = GTHUMB_STOCK_SEARCH;
+	else
+		stock_id = GTK_STOCK_OPEN;
+
+	g_free (home_uri);
+
+	return stock_id;
+}
+
+
+GdkPixbuf *
+get_icon_for_uri (GtkWidget  *widget,
+		  const char *uri)
+{
+	const char *stock_id = NULL;
+	char       *home_uri;
+	int         menu_size;
+
+	menu_size = get_folder_pixbuf_size_for_list (widget);
+
+	home_uri = g_strconcat ("file://", g_get_home_dir (), NULL);
+	if (strcmp (uri, g_get_home_dir ()) == 0) 
+		return get_fs_icon (ICON_NAME_HOME, menu_size);
+	else if (strcmp (uri, home_uri) == 0) 
+		return get_fs_icon (ICON_NAME_HOME, menu_size);
+	g_free (home_uri);
+
+	if ((strcmp (uri, "file://") == 0) || (strcmp (uri, "/") == 0)) 
+		return get_fs_icon (ICON_NAME_HARDDISK, menu_size);
+
+	if (folder_is_film (uri))
+		stock_id = GTHUMB_STOCK_FILM;
+	else if (uri_scheme_is_catalog (uri)) 
+		stock_id = GTHUMB_STOCK_CATALOG;
+	else if (uri_scheme_is_search (uri))	
+		stock_id = GTHUMB_STOCK_SEARCH;
+
+	if (stock_id != NULL)
+		return gtk_widget_render_icon (widget, stock_id, GTK_ICON_SIZE_MENU, NULL); 
+	else
+		return get_fs_icon (ICON_NAME_DIRECTORY, menu_size);
+}
